@@ -3,25 +3,26 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.2 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
+ * Please see the License for the specific language governing rights and 
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
 
+#include <IOKit/usb/IOUSBLog.h>
+
 #include "AppleUSBUHCI.h"
 
 
@@ -37,6 +38,7 @@ AppleUSBUHCI::UIMCreateInterruptTransfer(
                                          short				direction)
 {
     // deprecated
+	USBLog(1, "AppleUSBUHCI::UIMCreateInterruptTransfer - deprecated");
     return kIOReturnBadArgument;
 }
 
@@ -52,6 +54,7 @@ AppleUSBUHCI::UIMCreateBulkTransfer(
                                     short				direction)
 {
     // deprecated
+	USBLog(1, "AppleUSBUHCI::UIMCreateBulkTransfer - deprecated");
     return kIOReturnBadArgument;
 }
 
@@ -66,6 +69,7 @@ AppleUSBUHCI::UIMCreateControlTransfer(
                                        short				direction)
 {
     // deprecated
+	USBLog(1, "AppleUSBUHCI::UIMCreateControlTransfer - deprecated");
     return kIOReturnBadArgument;
 }
 
@@ -81,6 +85,7 @@ AppleUSBUHCI::UIMCreateControlTransfer(
                                        short				direction)
 {
     // deprecated
+	USBLog(1, "AppleUSBUHCI::UIMCreateControlTransfer - deprecated");
     return kIOReturnBadArgument;
 }
 
