@@ -3,19 +3,18 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.2 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
+ * Please see the License for the specific language governing rights and 
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -142,9 +141,12 @@ enum {
     kUHCI_FRAME_T   =   0x00000001
 };
 
-/* Transaction descriptor. */
+// Transfer descriptor
+typedef struct UHCITransferDescriptorShared
+UHCITransferDescriptorShared,
+*UHCITransferDescriptorSharedPtr;
 
-struct UHCITD {
+struct UHCITransferDescriptorShared {
     volatile UInt32  link;
     volatile UInt32  ctrlStatus;
     volatile UInt32  token;
@@ -190,11 +192,16 @@ enum {
 #define UHCI_TD_GET_MAXLEN(n)   ((((n) >> 21) + 1) & 0x7FF)
 #define UHCI_TD_SET_MAXLEN(n)   ((((n) - 1) & 0x7FF) << 21)
 
-/* Queue head. */
 
-struct UHCIQH {
+// Queue head
+typedef struct UHCIQueueHeadShared
+UHCIQueueHeadShared,
+*UHCIQueueHeadSharedPtr;
+
+struct UHCIQueueHeadShared {
     volatile UInt32  hlink;
     volatile UInt32  elink;
+	UInt32 			 pad[2];				// 4358445 - need to make sure these get 16 byte aligned
 };
 
 enum {
@@ -223,6 +230,15 @@ enum {
     kUHCI_QH_REMOVE_DELAY = 5
 };
 
+enum
+{
+    kUHCIPageSize			= 4096,
+};
+#define kUHCIPageOffsetMask	( kUHCIPageSize - 1 )		// mask off just the offset bits (low 12)
+#define kUHCIPageMask 		(~(kUHCIPageOffsetMask))	// mask off just the page number (high 20)
+#define kUHCIPtrMask		( 0xFFFFFFF0 )				// mask for list element pointers
+
+
 #endif /*  _IOKIT_UHCI_H_ */
 
 
