@@ -170,7 +170,7 @@ pid_t newPID;
      newPID = fork();
      if(newPID == 0)
      {
-        execlp(name, name, vendorStr, prodStr, 0);
+        execlp(name, name, vendorStr, prodStr, NULL);
         printf("Failed to exec new driver (err = %d)\n", errno);
     }
      else if(newPID == -1)
@@ -396,7 +396,8 @@ void RawDeviceAdded(void *refCon, io_iterator_t iterator)
             
         // I have the device plugin, I need the device interface
         res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID), (LPVOID)&dev);
-        (*plugInInterface)->Release(plugInInterface);			// done with this
+        IODestroyPlugInInterface(plugInInterface);			// done with this
+		
         if (res || !dev)
         {
             printf("couldn't create a device interface (%08x)\n", (int) res);
@@ -484,7 +485,8 @@ void NewDeviceAdded(void *refCon, io_iterator_t iterator)
             
         // I have the device plugin, I need the device interface
         res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID), (LPVOID)&dev);
-        (*plugInInterface)->Release(plugInInterface);			// done with this
+        IODestroyPlugInInterface(plugInInterface);			// done with this
+		
         if (res || !dev)
         {
             printf("couldn't create a device interface (%08x)\n", (int) res);
@@ -699,4 +701,4 @@ int main (int argc, const char *argv[])
         
     // We should never get here
     return 0;
-}
\ No newline at end of file
+}
