@@ -2,6 +2,8 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
+ * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -91,13 +93,21 @@ IOUSBNub::USBCompareProperty( OSDictionary   * matching, const char     * key )
     // We return success iff we match the key in the dictionary with the key in
     // the property table.
     //
-    OSObject 	* value;
+    OSObject 	*value;
     bool		matches;
+	OSObject	*myProperty = NULL;
 
     value = matching->getObject( key );
-
+	
     if( value)
-        matches = value->isEqualTo( getProperty( key ));
+	{
+		myProperty = copyProperty(key);
+		if (myProperty)
+		{
+			matches = value->isEqualTo( myProperty);
+			myProperty->release();
+		}
+	}
     else
         matches = false;
 
