@@ -1,21 +1,20 @@
 /*
- * Copyright (c) 1998-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2006 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.2 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
+ * Please see the License for the specific language governing rights and 
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -55,6 +54,8 @@ protected:
         IOReturn	_correctStatus;		
 		IOUSBDevice *	_device;	// Remember containing device for clearing TTs
 		UInt8			_speed;
+		IOUSBInterface	* _interface;
+		bool			_crossEndianCompatible;
     };
     ExpansionData * _expansionData;
     
@@ -69,7 +70,7 @@ public:
 		virtual bool InitToEndpoint(const IOUSBEndpointDescriptor *endpoint, UInt8 speed,
 									USBDeviceAddress address, IOUSBController * controller);
 	
-    // The following method is obsolete
+    // The following 2 methods are obsolete
     //
     static IOUSBPipe *ToEndpoint(const IOUSBEndpointDescriptor *endpoint, UInt8 speed,
                                  USBDeviceAddress address, IOUSBController * controller);
@@ -77,6 +78,9 @@ public:
     static IOUSBPipe *ToEndpoint(const IOUSBEndpointDescriptor *endpoint,
                                  IOUSBDevice * device, IOUSBController * controller);
 	
+    static IOUSBPipe *ToEndpoint(const IOUSBEndpointDescriptor *endpoint,
+                                 IOUSBDevice * device, IOUSBController * controller, IOUSBInterface *interface);
+	
     // Controlling pipe state
     /*!
         @function GetStatus
